// Copyright (c) Microsoft Corporation
// All rights reserved. 
//
// Licensed under the Apache License, Version 2.0 (the ""License""); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
//
// THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//
// See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.

#define NTDDI_VERSION 0x06010000

#include <v8.h>
#include <string>
#include <node_object_wrap.h>
#include <ppltasks.h>
#include "CollectionsConverter.h"
#include "CollectionsWrap.h"
#include "node-async.h"
#include "NodeRtUtils.h"
#include "OpaqueWrapper.h"
#include "WrapperBase.h"

#using <Windows.WinMD>

// this undefs fixes the issues of compiling Windows.Data.Json, Windows.Storag.FileProperties, and Windows.Stroage.Search
// Some of the node header files brings windows definitions with the same names as some of the WinRT methods
#undef DocumentProperties
#undef GetObject
#undef CreateEvent
#undef FindText
#undef SendMessage

const char* REGISTRATION_TOKEN_MAP_PROPERTY_NAME = "__registrationTokenMap__";

using namespace v8;
using namespace node;
using namespace concurrency;

namespace NodeRT { namespace Windows { namespace UI { namespace Notifications { 
  v8::Handle<v8::Value> WrapTileUpdater(::Windows::UI::Notifications::TileUpdater^ wintRtInstance);
  ::Windows::UI::Notifications::TileUpdater^ UnwrapTileUpdater(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapTileNotification(::Windows::UI::Notifications::TileNotification^ wintRtInstance);
  ::Windows::UI::Notifications::TileNotification^ UnwrapTileNotification(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapScheduledTileNotification(::Windows::UI::Notifications::ScheduledTileNotification^ wintRtInstance);
  ::Windows::UI::Notifications::ScheduledTileNotification^ UnwrapScheduledTileNotification(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapBadgeUpdater(::Windows::UI::Notifications::BadgeUpdater^ wintRtInstance);
  ::Windows::UI::Notifications::BadgeUpdater^ UnwrapBadgeUpdater(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapBadgeNotification(::Windows::UI::Notifications::BadgeNotification^ wintRtInstance);
  ::Windows::UI::Notifications::BadgeNotification^ UnwrapBadgeNotification(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapToastNotifier(::Windows::UI::Notifications::ToastNotifier^ wintRtInstance);
  ::Windows::UI::Notifications::ToastNotifier^ UnwrapToastNotifier(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapToastNotification(::Windows::UI::Notifications::ToastNotification^ wintRtInstance);
  ::Windows::UI::Notifications::ToastNotification^ UnwrapToastNotification(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapScheduledToastNotification(::Windows::UI::Notifications::ScheduledToastNotification^ wintRtInstance);
  ::Windows::UI::Notifications::ScheduledToastNotification^ UnwrapScheduledToastNotification(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapToastDismissedEventArgs(::Windows::UI::Notifications::ToastDismissedEventArgs^ wintRtInstance);
  ::Windows::UI::Notifications::ToastDismissedEventArgs^ UnwrapToastDismissedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapToastFailedEventArgs(::Windows::UI::Notifications::ToastFailedEventArgs^ wintRtInstance);
  ::Windows::UI::Notifications::ToastFailedEventArgs^ UnwrapToastFailedEventArgs(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapTileUpdateManager(::Windows::UI::Notifications::TileUpdateManager^ wintRtInstance);
  ::Windows::UI::Notifications::TileUpdateManager^ UnwrapTileUpdateManager(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapBadgeUpdateManager(::Windows::UI::Notifications::BadgeUpdateManager^ wintRtInstance);
  ::Windows::UI::Notifications::BadgeUpdateManager^ UnwrapBadgeUpdateManager(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapToastNotificationManager(::Windows::UI::Notifications::ToastNotificationManager^ wintRtInstance);
  ::Windows::UI::Notifications::ToastNotificationManager^ UnwrapToastNotificationManager(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapToastActivatedEventArgs(::Windows::UI::Notifications::ToastActivatedEventArgs^ wintRtInstance);
  ::Windows::UI::Notifications::ToastActivatedEventArgs^ UnwrapToastActivatedEventArgs(Handle<Value> value);
  


  static v8::Handle<v8::Value> InitNotificationSettingEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("NotificationSetting"), enumObject);

    enumObject->Set(String::NewSymbol("enabled"), Integer::New(0));
    enumObject->Set(String::NewSymbol("disabledForApplication"), Integer::New(1));
    enumObject->Set(String::NewSymbol("disabledForUser"), Integer::New(2));
    enumObject->Set(String::NewSymbol("disabledByGroupPolicy"), Integer::New(3));
    enumObject->Set(String::NewSymbol("disabledByManifest"), Integer::New(4));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitToastDismissalReasonEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ToastDismissalReason"), enumObject);

    enumObject->Set(String::NewSymbol("userCanceled"), Integer::New(0));
    enumObject->Set(String::NewSymbol("applicationHidden"), Integer::New(1));
    enumObject->Set(String::NewSymbol("timedOut"), Integer::New(2));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitBadgeTemplateTypeEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("BadgeTemplateType"), enumObject);

    enumObject->Set(String::NewSymbol("badgeGlyph"), Integer::New(0));
    enumObject->Set(String::NewSymbol("badgeNumber"), Integer::New(1));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitTileTemplateTypeEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("TileTemplateType"), enumObject);

    enumObject->Set(String::NewSymbol("tileSquareImage"), Integer::New(0));
    enumObject->Set(String::NewSymbol("tileSquareBlock"), Integer::New(1));
    enumObject->Set(String::NewSymbol("tileSquareText01"), Integer::New(2));
    enumObject->Set(String::NewSymbol("tileSquareText02"), Integer::New(3));
    enumObject->Set(String::NewSymbol("tileSquareText03"), Integer::New(4));
    enumObject->Set(String::NewSymbol("tileSquareText04"), Integer::New(5));
    enumObject->Set(String::NewSymbol("tileSquarePeekImageAndText01"), Integer::New(6));
    enumObject->Set(String::NewSymbol("tileSquarePeekImageAndText02"), Integer::New(7));
    enumObject->Set(String::NewSymbol("tileSquarePeekImageAndText03"), Integer::New(8));
    enumObject->Set(String::NewSymbol("tileSquarePeekImageAndText04"), Integer::New(9));
    enumObject->Set(String::NewSymbol("tileWideImage"), Integer::New(10));
    enumObject->Set(String::NewSymbol("tileWideImageCollection"), Integer::New(11));
    enumObject->Set(String::NewSymbol("tileWideImageAndText01"), Integer::New(12));
    enumObject->Set(String::NewSymbol("tileWideImageAndText02"), Integer::New(13));
    enumObject->Set(String::NewSymbol("tileWideBlockAndText01"), Integer::New(14));
    enumObject->Set(String::NewSymbol("tileWideBlockAndText02"), Integer::New(15));
    enumObject->Set(String::NewSymbol("tileWidePeekImageCollection01"), Integer::New(16));
    enumObject->Set(String::NewSymbol("tileWidePeekImageCollection02"), Integer::New(17));
    enumObject->Set(String::NewSymbol("tileWidePeekImageCollection03"), Integer::New(18));
    enumObject->Set(String::NewSymbol("tileWidePeekImageCollection04"), Integer::New(19));
    enumObject->Set(String::NewSymbol("tileWidePeekImageCollection05"), Integer::New(20));
    enumObject->Set(String::NewSymbol("tileWidePeekImageCollection06"), Integer::New(21));
    enumObject->Set(String::NewSymbol("tileWidePeekImageAndText01"), Integer::New(22));
    enumObject->Set(String::NewSymbol("tileWidePeekImageAndText02"), Integer::New(23));
    enumObject->Set(String::NewSymbol("tileWidePeekImage01"), Integer::New(24));
    enumObject->Set(String::NewSymbol("tileWidePeekImage02"), Integer::New(25));
    enumObject->Set(String::NewSymbol("tileWidePeekImage03"), Integer::New(26));
    enumObject->Set(String::NewSymbol("tileWidePeekImage04"), Integer::New(27));
    enumObject->Set(String::NewSymbol("tileWidePeekImage05"), Integer::New(28));
    enumObject->Set(String::NewSymbol("tileWidePeekImage06"), Integer::New(29));
    enumObject->Set(String::NewSymbol("tileWideSmallImageAndText01"), Integer::New(30));
    enumObject->Set(String::NewSymbol("tileWideSmallImageAndText02"), Integer::New(31));
    enumObject->Set(String::NewSymbol("tileWideSmallImageAndText03"), Integer::New(32));
    enumObject->Set(String::NewSymbol("tileWideSmallImageAndText04"), Integer::New(33));
    enumObject->Set(String::NewSymbol("tileWideSmallImageAndText05"), Integer::New(34));
    enumObject->Set(String::NewSymbol("tileWideText01"), Integer::New(35));
    enumObject->Set(String::NewSymbol("tileWideText02"), Integer::New(36));
    enumObject->Set(String::NewSymbol("tileWideText03"), Integer::New(37));
    enumObject->Set(String::NewSymbol("tileWideText04"), Integer::New(38));
    enumObject->Set(String::NewSymbol("tileWideText05"), Integer::New(39));
    enumObject->Set(String::NewSymbol("tileWideText06"), Integer::New(40));
    enumObject->Set(String::NewSymbol("tileWideText07"), Integer::New(41));
    enumObject->Set(String::NewSymbol("tileWideText08"), Integer::New(42));
    enumObject->Set(String::NewSymbol("tileWideText09"), Integer::New(43));
    enumObject->Set(String::NewSymbol("tileWideText10"), Integer::New(44));
    enumObject->Set(String::NewSymbol("tileWideText11"), Integer::New(45));
    enumObject->Set(String::NewSymbol("tileSquare150x150Image"), Integer::New(46));
    enumObject->Set(String::NewSymbol("tileSquare150x150Block"), Integer::New(47));
    enumObject->Set(String::NewSymbol("tileSquare150x150Text01"), Integer::New(48));
    enumObject->Set(String::NewSymbol("tileSquare150x150Text02"), Integer::New(49));
    enumObject->Set(String::NewSymbol("tileSquare150x150Text03"), Integer::New(50));
    enumObject->Set(String::NewSymbol("tileSquare150x150Text04"), Integer::New(51));
    enumObject->Set(String::NewSymbol("tileSquare150x150PeekImageAndText01"), Integer::New(52));
    enumObject->Set(String::NewSymbol("tileSquare150x150PeekImageAndText02"), Integer::New(53));
    enumObject->Set(String::NewSymbol("tileSquare150x150PeekImageAndText03"), Integer::New(54));
    enumObject->Set(String::NewSymbol("tileSquare150x150PeekImageAndText04"), Integer::New(55));
    enumObject->Set(String::NewSymbol("tileWide310x150Image"), Integer::New(56));
    enumObject->Set(String::NewSymbol("tileWide310x150ImageCollection"), Integer::New(57));
    enumObject->Set(String::NewSymbol("tileWide310x150ImageAndText01"), Integer::New(58));
    enumObject->Set(String::NewSymbol("tileWide310x150ImageAndText02"), Integer::New(59));
    enumObject->Set(String::NewSymbol("tileWide310x150BlockAndText01"), Integer::New(60));
    enumObject->Set(String::NewSymbol("tileWide310x150BlockAndText02"), Integer::New(61));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImageCollection01"), Integer::New(62));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImageCollection02"), Integer::New(63));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImageCollection03"), Integer::New(64));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImageCollection04"), Integer::New(65));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImageCollection05"), Integer::New(66));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImageCollection06"), Integer::New(67));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImageAndText01"), Integer::New(68));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImageAndText02"), Integer::New(69));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImage01"), Integer::New(70));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImage02"), Integer::New(71));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImage03"), Integer::New(72));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImage04"), Integer::New(73));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImage05"), Integer::New(74));
    enumObject->Set(String::NewSymbol("tileWide310x150PeekImage06"), Integer::New(75));
    enumObject->Set(String::NewSymbol("tileWide310x150SmallImageAndText01"), Integer::New(76));
    enumObject->Set(String::NewSymbol("tileWide310x150SmallImageAndText02"), Integer::New(77));
    enumObject->Set(String::NewSymbol("tileWide310x150SmallImageAndText03"), Integer::New(78));
    enumObject->Set(String::NewSymbol("tileWide310x150SmallImageAndText04"), Integer::New(79));
    enumObject->Set(String::NewSymbol("tileWide310x150SmallImageAndText05"), Integer::New(80));
    enumObject->Set(String::NewSymbol("tileWide310x150Text01"), Integer::New(81));
    enumObject->Set(String::NewSymbol("tileWide310x150Text02"), Integer::New(82));
    enumObject->Set(String::NewSymbol("tileWide310x150Text03"), Integer::New(83));
    enumObject->Set(String::NewSymbol("tileWide310x150Text04"), Integer::New(84));
    enumObject->Set(String::NewSymbol("tileWide310x150Text05"), Integer::New(85));
    enumObject->Set(String::NewSymbol("tileWide310x150Text06"), Integer::New(86));
    enumObject->Set(String::NewSymbol("tileWide310x150Text07"), Integer::New(87));
    enumObject->Set(String::NewSymbol("tileWide310x150Text08"), Integer::New(88));
    enumObject->Set(String::NewSymbol("tileWide310x150Text09"), Integer::New(89));
    enumObject->Set(String::NewSymbol("tileWide310x150Text10"), Integer::New(90));
    enumObject->Set(String::NewSymbol("tileWide310x150Text11"), Integer::New(91));
    enumObject->Set(String::NewSymbol("tileSquare310x310BlockAndText01"), Integer::New(92));
    enumObject->Set(String::NewSymbol("tileSquare310x310BlockAndText02"), Integer::New(93));
    enumObject->Set(String::NewSymbol("tileSquare310x310Image"), Integer::New(94));
    enumObject->Set(String::NewSymbol("tileSquare310x310ImageAndText01"), Integer::New(95));
    enumObject->Set(String::NewSymbol("tileSquare310x310ImageAndText02"), Integer::New(96));
    enumObject->Set(String::NewSymbol("tileSquare310x310ImageAndTextOverlay01"), Integer::New(97));
    enumObject->Set(String::NewSymbol("tileSquare310x310ImageAndTextOverlay02"), Integer::New(98));
    enumObject->Set(String::NewSymbol("tileSquare310x310ImageAndTextOverlay03"), Integer::New(99));
    enumObject->Set(String::NewSymbol("tileSquare310x310ImageCollectionAndText01"), Integer::New(100));
    enumObject->Set(String::NewSymbol("tileSquare310x310ImageCollectionAndText02"), Integer::New(101));
    enumObject->Set(String::NewSymbol("tileSquare310x310ImageCollection"), Integer::New(102));
    enumObject->Set(String::NewSymbol("tileSquare310x310SmallImagesAndTextList01"), Integer::New(103));
    enumObject->Set(String::NewSymbol("tileSquare310x310SmallImagesAndTextList02"), Integer::New(104));
    enumObject->Set(String::NewSymbol("tileSquare310x310SmallImagesAndTextList03"), Integer::New(105));
    enumObject->Set(String::NewSymbol("tileSquare310x310SmallImagesAndTextList04"), Integer::New(106));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text01"), Integer::New(107));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text02"), Integer::New(108));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text03"), Integer::New(109));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text04"), Integer::New(110));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text05"), Integer::New(111));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text06"), Integer::New(112));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text07"), Integer::New(113));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text08"), Integer::New(114));
    enumObject->Set(String::NewSymbol("tileSquare310x310TextList01"), Integer::New(115));
    enumObject->Set(String::NewSymbol("tileSquare310x310TextList02"), Integer::New(116));
    enumObject->Set(String::NewSymbol("tileSquare310x310TextList03"), Integer::New(117));
    enumObject->Set(String::NewSymbol("tileSquare310x310SmallImageAndText01"), Integer::New(118));
    enumObject->Set(String::NewSymbol("tileSquare310x310SmallImagesAndTextList05"), Integer::New(119));
    enumObject->Set(String::NewSymbol("tileSquare310x310Text09"), Integer::New(120));
    enumObject->Set(String::NewSymbol("tileSquare99x99IconWithBadge"), Integer::New(121));
    enumObject->Set(String::NewSymbol("tileSquare210x210IconWithBadge"), Integer::New(122));
    enumObject->Set(String::NewSymbol("tileWide432x210IconWithBadgeAndText"), Integer::New(123));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitToastTemplateTypeEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ToastTemplateType"), enumObject);

    enumObject->Set(String::NewSymbol("toastImageAndText01"), Integer::New(0));
    enumObject->Set(String::NewSymbol("toastImageAndText02"), Integer::New(1));
    enumObject->Set(String::NewSymbol("toastImageAndText03"), Integer::New(2));
    enumObject->Set(String::NewSymbol("toastImageAndText04"), Integer::New(3));
    enumObject->Set(String::NewSymbol("toastText01"), Integer::New(4));
    enumObject->Set(String::NewSymbol("toastText02"), Integer::New(5));
    enumObject->Set(String::NewSymbol("toastText03"), Integer::New(6));
    enumObject->Set(String::NewSymbol("toastText04"), Integer::New(7));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitPeriodicUpdateRecurrenceEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("PeriodicUpdateRecurrence"), enumObject);

    enumObject->Set(String::NewSymbol("halfHour"), Integer::New(0));
    enumObject->Set(String::NewSymbol("hour"), Integer::New(1));
    enumObject->Set(String::NewSymbol("sixHours"), Integer::New(2));
    enumObject->Set(String::NewSymbol("twelveHours"), Integer::New(3));
    enumObject->Set(String::NewSymbol("daily"), Integer::New(4));

    return scope.Close(Undefined());
  }



  
  class TileUpdater : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("TileUpdater"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("update"), FunctionTemplate::New(Update)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("clear"), FunctionTemplate::New(Clear)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("enableNotificationQueue"), FunctionTemplate::New(EnableNotificationQueue)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addToSchedule"), FunctionTemplate::New(AddToSchedule)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeFromSchedule"), FunctionTemplate::New(RemoveFromSchedule)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getScheduledTileNotifications"), FunctionTemplate::New(GetScheduledTileNotifications)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("startPeriodicUpdate"), FunctionTemplate::New(StartPeriodicUpdate)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("stopPeriodicUpdate"), FunctionTemplate::New(StopPeriodicUpdate)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("startPeriodicUpdateBatch"), FunctionTemplate::New(StartPeriodicUpdateBatch)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("enableNotificationQueueForSquare150x150"), FunctionTemplate::New(EnableNotificationQueueForSquare150x150)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("enableNotificationQueueForWide310x150"), FunctionTemplate::New(EnableNotificationQueueForWide310x150)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("enableNotificationQueueForSquare310x310"), FunctionTemplate::New(EnableNotificationQueueForSquare310x310)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("setting"), SettingGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("TileUpdater"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    TileUpdater(::Windows::UI::Notifications::TileUpdater^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::TileUpdater^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::TileUpdater^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      TileUpdater *wrapperInstance = new TileUpdater(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> Update(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileNotification^>(args[0]))
      {
        try
        {
          ::Windows::UI::Notifications::TileNotification^ arg0 = UnwrapTileNotification(args[0]);
          
          wrapper->_instance->Update(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Clear(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->Clear();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> EnableNotificationQueue(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 1
        && args[0]->IsBoolean())
      {
        try
        {
          bool arg0 = args[0]->BooleanValue();
          
          wrapper->_instance->EnableNotificationQueue(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> AddToSchedule(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(args[0]))
      {
        try
        {
          ::Windows::UI::Notifications::ScheduledTileNotification^ arg0 = UnwrapScheduledTileNotification(args[0]);
          
          wrapper->_instance->AddToSchedule(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RemoveFromSchedule(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(args[0]))
      {
        try
        {
          ::Windows::UI::Notifications::ScheduledTileNotification^ arg0 = UnwrapScheduledTileNotification(args[0]);
          
          wrapper->_instance->RemoveFromSchedule(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetScheduledTileNotifications(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Foundation::Collections::IVectorView<::Windows::UI::Notifications::ScheduledTileNotification^>^ result;
          result = wrapper->_instance->GetScheduledTileNotifications();
          return scope.Close(NodeRT::Collections::VectorViewWrapper<::Windows::UI::Notifications::ScheduledTileNotification^>::CreateVectorViewWrapper(result, 
            [](::Windows::UI::Notifications::ScheduledTileNotification^ val) -> Handle<Value> {
              return WrapScheduledTileNotification(val);
            },
            [](Handle<Value> value) -> bool {
              return NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(value);
            },
            [](Handle<Value> value) -> ::Windows::UI::Notifications::ScheduledTileNotification^ {
              return UnwrapScheduledTileNotification(value);
            }
          ));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> StartPeriodicUpdate(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(args[0])
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Uri^ arg0 = dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(args[0]));
          ::Windows::UI::Notifications::PeriodicUpdateRecurrence arg1 = static_cast<::Windows::UI::Notifications::PeriodicUpdateRecurrence>(args[1]->Int32Value());
          
          wrapper->_instance->StartPeriodicUpdate(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(args[0])
        && args[1]->IsDate()
        && args[2]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Uri^ arg0 = dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(args[0]));
          ::Windows::Foundation::DateTime arg1 = NodeRT::Utils::DateTimeFromJSDate(args[1]);
          ::Windows::UI::Notifications::PeriodicUpdateRecurrence arg2 = static_cast<::Windows::UI::Notifications::PeriodicUpdateRecurrence>(args[2]->Int32Value());
          
          wrapper->_instance->StartPeriodicUpdate(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> StopPeriodicUpdate(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->StopPeriodicUpdate();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> StartPeriodicUpdateBatch(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 2
        && (NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IIterable<::Windows::Foundation::Uri^>^>(args[0]) || args[0]->IsArray())
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Collections::IIterable<::Windows::Foundation::Uri^>^ arg0 = 
            [] (v8::Handle<v8::Value> value) -> ::Windows::Foundation::Collections::IIterable<::Windows::Foundation::Uri^>^
            {
              if (value->IsArray())
              {
                return NodeRT::Collections::JsArrayToWinrtVector<::Windows::Foundation::Uri^>(value.As<Array>(), 
                 [](Handle<Value> value) -> bool {
                   return NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(value);
                 },
                 [](Handle<Value> value) -> ::Windows::Foundation::Uri^ {
                   return dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(value));
                 }
                );
              }
              else
              {
                return dynamic_cast<::Windows::Foundation::Collections::IIterable<::Windows::Foundation::Uri^>^>(NodeRT::Utils::GetObjectInstance(value));
              }
            } (args[0]);
          ::Windows::UI::Notifications::PeriodicUpdateRecurrence arg1 = static_cast<::Windows::UI::Notifications::PeriodicUpdateRecurrence>(args[1]->Int32Value());
          
          wrapper->_instance->StartPeriodicUpdateBatch(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && (NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Collections::IIterable<::Windows::Foundation::Uri^>^>(args[0]) || args[0]->IsArray())
        && args[1]->IsDate()
        && args[2]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Collections::IIterable<::Windows::Foundation::Uri^>^ arg0 = 
            [] (v8::Handle<v8::Value> value) -> ::Windows::Foundation::Collections::IIterable<::Windows::Foundation::Uri^>^
            {
              if (value->IsArray())
              {
                return NodeRT::Collections::JsArrayToWinrtVector<::Windows::Foundation::Uri^>(value.As<Array>(), 
                 [](Handle<Value> value) -> bool {
                   return NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(value);
                 },
                 [](Handle<Value> value) -> ::Windows::Foundation::Uri^ {
                   return dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(value));
                 }
                );
              }
              else
              {
                return dynamic_cast<::Windows::Foundation::Collections::IIterable<::Windows::Foundation::Uri^>^>(NodeRT::Utils::GetObjectInstance(value));
              }
            } (args[0]);
          ::Windows::Foundation::DateTime arg1 = NodeRT::Utils::DateTimeFromJSDate(args[1]);
          ::Windows::UI::Notifications::PeriodicUpdateRecurrence arg2 = static_cast<::Windows::UI::Notifications::PeriodicUpdateRecurrence>(args[2]->Int32Value());
          
          wrapper->_instance->StartPeriodicUpdateBatch(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> EnableNotificationQueueForSquare150x150(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 1
        && args[0]->IsBoolean())
      {
        try
        {
          bool arg0 = args[0]->BooleanValue();
          
          wrapper->_instance->EnableNotificationQueueForSquare150x150(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> EnableNotificationQueueForWide310x150(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 1
        && args[0]->IsBoolean())
      {
        try
        {
          bool arg0 = args[0]->BooleanValue();
          
          wrapper->_instance->EnableNotificationQueueForWide310x150(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> EnableNotificationQueueForSquare310x310(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(args.This());

      if (args.Length() == 1
        && args[0]->IsBoolean())
      {
        try
        {
          bool arg0 = args[0]->BooleanValue();
          
          wrapper->_instance->EnableNotificationQueueForSquare310x310(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> SettingGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdater^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      TileUpdater *wrapper = TileUpdater::Unwrap<TileUpdater>(info.This());

      try 
      {
        ::Windows::UI::Notifications::NotificationSetting result = wrapper->_instance->Setting;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::TileUpdater^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapTileUpdater(::Windows::UI::Notifications::TileUpdater^ wintRtInstance);
    friend ::Windows::UI::Notifications::TileUpdater^ UnwrapTileUpdater(Handle<Value> value);
    friend bool IsTileUpdaterWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> TileUpdater::s_constructorTemplate;

  v8::Handle<v8::Value> WrapTileUpdater(::Windows::UI::Notifications::TileUpdater^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(TileUpdater::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::TileUpdater^ UnwrapTileUpdater(Handle<Value> value)
  {
     return TileUpdater::Unwrap<TileUpdater>(value.As<Object>())->_instance;
  }

  void InitTileUpdater(Handle<Object> exports)
  {
    TileUpdater::Init(exports);
  }

  class TileNotification : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("TileNotification"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("tag"), TagGetter, TagSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("expirationTime"), ExpirationTimeGetter, ExpirationTimeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("content"), ContentGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("TileNotification"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    TileNotification(::Windows::UI::Notifications::TileNotification^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::TileNotification^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileNotification^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::TileNotification^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Data::Xml::Dom::XmlDocument^>(args[0]))
      {
        try
        {
          ::Windows::Data::Xml::Dom::XmlDocument^ arg0 = dynamic_cast<::Windows::Data::Xml::Dom::XmlDocument^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          winRtInstance = ref new ::Windows::UI::Notifications::TileNotification(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      TileNotification *wrapperInstance = new TileNotification(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> TagGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      TileNotification *wrapper = TileNotification::Unwrap<TileNotification>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Tag;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void TagSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileNotification^>(info.This()))
      {
        return;
      }

      TileNotification *wrapper = TileNotification::Unwrap<TileNotification>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Tag = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ExpirationTimeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      TileNotification *wrapper = TileNotification::Unwrap<TileNotification>(info.This());

      try 
      {
        ::Platform::IBox<::Windows::Foundation::DateTime>^ result = wrapper->_instance->ExpirationTime;
        return scope.Close(result ? NodeRT::Utils::DateTimeToJS(result->Value): static_cast<Handle<Value>>(Undefined()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ExpirationTimeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsDate())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileNotification^>(info.This()))
      {
        return;
      }

      TileNotification *wrapper = TileNotification::Unwrap<TileNotification>(info.This());

      try 
      {
        
        ::Platform::IBox<::Windows::Foundation::DateTime>^ winRtValue = ref new ::Platform::Box<::Windows::Foundation::DateTime>(NodeRT::Utils::DateTimeFromJSDate(value));

        wrapper->_instance->ExpirationTime = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContentGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      TileNotification *wrapper = TileNotification::Unwrap<TileNotification>(info.This());

      try 
      {
        ::Windows::Data::Xml::Dom::XmlDocument^ result = wrapper->_instance->Content;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Data.Xml.Dom", "XmlDocument", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::TileNotification^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapTileNotification(::Windows::UI::Notifications::TileNotification^ wintRtInstance);
    friend ::Windows::UI::Notifications::TileNotification^ UnwrapTileNotification(Handle<Value> value);
    friend bool IsTileNotificationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> TileNotification::s_constructorTemplate;

  v8::Handle<v8::Value> WrapTileNotification(::Windows::UI::Notifications::TileNotification^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(TileNotification::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::TileNotification^ UnwrapTileNotification(Handle<Value> value)
  {
     return TileNotification::Unwrap<TileNotification>(value.As<Object>())->_instance;
  }

  void InitTileNotification(Handle<Object> exports)
  {
    TileNotification::Init(exports);
  }

  class ScheduledTileNotification : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ScheduledTileNotification"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("tag"), TagGetter, TagSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("id"), IdGetter, IdSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("expirationTime"), ExpirationTimeGetter, ExpirationTimeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("content"), ContentGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("deliveryTime"), DeliveryTimeGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ScheduledTileNotification"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ScheduledTileNotification(::Windows::UI::Notifications::ScheduledTileNotification^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::ScheduledTileNotification^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::ScheduledTileNotification^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Data::Xml::Dom::XmlDocument^>(args[0])
        && args[1]->IsDate())
      {
        try
        {
          ::Windows::Data::Xml::Dom::XmlDocument^ arg0 = dynamic_cast<::Windows::Data::Xml::Dom::XmlDocument^>(NodeRT::Utils::GetObjectInstance(args[0]));
          ::Windows::Foundation::DateTime arg1 = NodeRT::Utils::DateTimeFromJSDate(args[1]);
          
          winRtInstance = ref new ::Windows::UI::Notifications::ScheduledTileNotification(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ScheduledTileNotification *wrapperInstance = new ScheduledTileNotification(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> TagGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledTileNotification *wrapper = ScheduledTileNotification::Unwrap<ScheduledTileNotification>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Tag;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void TagSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(info.This()))
      {
        return;
      }

      ScheduledTileNotification *wrapper = ScheduledTileNotification::Unwrap<ScheduledTileNotification>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Tag = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> IdGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledTileNotification *wrapper = ScheduledTileNotification::Unwrap<ScheduledTileNotification>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Id;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IdSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(info.This()))
      {
        return;
      }

      ScheduledTileNotification *wrapper = ScheduledTileNotification::Unwrap<ScheduledTileNotification>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Id = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ExpirationTimeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledTileNotification *wrapper = ScheduledTileNotification::Unwrap<ScheduledTileNotification>(info.This());

      try 
      {
        ::Platform::IBox<::Windows::Foundation::DateTime>^ result = wrapper->_instance->ExpirationTime;
        return scope.Close(result ? NodeRT::Utils::DateTimeToJS(result->Value): static_cast<Handle<Value>>(Undefined()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ExpirationTimeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsDate())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(info.This()))
      {
        return;
      }

      ScheduledTileNotification *wrapper = ScheduledTileNotification::Unwrap<ScheduledTileNotification>(info.This());

      try 
      {
        
        ::Platform::IBox<::Windows::Foundation::DateTime>^ winRtValue = ref new ::Platform::Box<::Windows::Foundation::DateTime>(NodeRT::Utils::DateTimeFromJSDate(value));

        wrapper->_instance->ExpirationTime = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContentGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledTileNotification *wrapper = ScheduledTileNotification::Unwrap<ScheduledTileNotification>(info.This());

      try 
      {
        ::Windows::Data::Xml::Dom::XmlDocument^ result = wrapper->_instance->Content;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Data.Xml.Dom", "XmlDocument", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DeliveryTimeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledTileNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledTileNotification *wrapper = ScheduledTileNotification::Unwrap<ScheduledTileNotification>(info.This());

      try 
      {
        ::Windows::Foundation::DateTime result = wrapper->_instance->DeliveryTime;
        return scope.Close(NodeRT::Utils::DateTimeToJS(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::ScheduledTileNotification^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapScheduledTileNotification(::Windows::UI::Notifications::ScheduledTileNotification^ wintRtInstance);
    friend ::Windows::UI::Notifications::ScheduledTileNotification^ UnwrapScheduledTileNotification(Handle<Value> value);
    friend bool IsScheduledTileNotificationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ScheduledTileNotification::s_constructorTemplate;

  v8::Handle<v8::Value> WrapScheduledTileNotification(::Windows::UI::Notifications::ScheduledTileNotification^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ScheduledTileNotification::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::ScheduledTileNotification^ UnwrapScheduledTileNotification(Handle<Value> value)
  {
     return ScheduledTileNotification::Unwrap<ScheduledTileNotification>(value.As<Object>())->_instance;
  }

  void InitScheduledTileNotification(Handle<Object> exports)
  {
    ScheduledTileNotification::Init(exports);
  }

  class BadgeUpdater : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("BadgeUpdater"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("update"), FunctionTemplate::New(Update)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("clear"), FunctionTemplate::New(Clear)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("startPeriodicUpdate"), FunctionTemplate::New(StartPeriodicUpdate)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("stopPeriodicUpdate"), FunctionTemplate::New(StopPeriodicUpdate)->GetFunction());
      
                        
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("BadgeUpdater"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    BadgeUpdater(::Windows::UI::Notifications::BadgeUpdater^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::BadgeUpdater^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeUpdater^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::BadgeUpdater^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      BadgeUpdater *wrapperInstance = new BadgeUpdater(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> Update(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      BadgeUpdater *wrapper = BadgeUpdater::Unwrap<BadgeUpdater>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeNotification^>(args[0]))
      {
        try
        {
          ::Windows::UI::Notifications::BadgeNotification^ arg0 = UnwrapBadgeNotification(args[0]);
          
          wrapper->_instance->Update(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Clear(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      BadgeUpdater *wrapper = BadgeUpdater::Unwrap<BadgeUpdater>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->Clear();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> StartPeriodicUpdate(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      BadgeUpdater *wrapper = BadgeUpdater::Unwrap<BadgeUpdater>(args.This());

      if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(args[0])
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Uri^ arg0 = dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(args[0]));
          ::Windows::UI::Notifications::PeriodicUpdateRecurrence arg1 = static_cast<::Windows::UI::Notifications::PeriodicUpdateRecurrence>(args[1]->Int32Value());
          
          wrapper->_instance->StartPeriodicUpdate(arg0, arg1);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 3
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Foundation::Uri^>(args[0])
        && args[1]->IsDate()
        && args[2]->IsInt32())
      {
        try
        {
          ::Windows::Foundation::Uri^ arg0 = dynamic_cast<::Windows::Foundation::Uri^>(NodeRT::Utils::GetObjectInstance(args[0]));
          ::Windows::Foundation::DateTime arg1 = NodeRT::Utils::DateTimeFromJSDate(args[1]);
          ::Windows::UI::Notifications::PeriodicUpdateRecurrence arg2 = static_cast<::Windows::UI::Notifications::PeriodicUpdateRecurrence>(args[2]->Int32Value());
          
          wrapper->_instance->StartPeriodicUpdate(arg0, arg1, arg2);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> StopPeriodicUpdate(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeUpdater^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      BadgeUpdater *wrapper = BadgeUpdater::Unwrap<BadgeUpdater>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          wrapper->_instance->StopPeriodicUpdate();
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }





  private:
    ::Windows::UI::Notifications::BadgeUpdater^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapBadgeUpdater(::Windows::UI::Notifications::BadgeUpdater^ wintRtInstance);
    friend ::Windows::UI::Notifications::BadgeUpdater^ UnwrapBadgeUpdater(Handle<Value> value);
    friend bool IsBadgeUpdaterWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> BadgeUpdater::s_constructorTemplate;

  v8::Handle<v8::Value> WrapBadgeUpdater(::Windows::UI::Notifications::BadgeUpdater^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(BadgeUpdater::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::BadgeUpdater^ UnwrapBadgeUpdater(Handle<Value> value)
  {
     return BadgeUpdater::Unwrap<BadgeUpdater>(value.As<Object>())->_instance;
  }

  void InitBadgeUpdater(Handle<Object> exports)
  {
    BadgeUpdater::Init(exports);
  }

  class BadgeNotification : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("BadgeNotification"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("expirationTime"), ExpirationTimeGetter, ExpirationTimeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("content"), ContentGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("BadgeNotification"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    BadgeNotification(::Windows::UI::Notifications::BadgeNotification^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::BadgeNotification^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeNotification^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::BadgeNotification^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Data::Xml::Dom::XmlDocument^>(args[0]))
      {
        try
        {
          ::Windows::Data::Xml::Dom::XmlDocument^ arg0 = dynamic_cast<::Windows::Data::Xml::Dom::XmlDocument^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          winRtInstance = ref new ::Windows::UI::Notifications::BadgeNotification(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      BadgeNotification *wrapperInstance = new BadgeNotification(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> ExpirationTimeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      BadgeNotification *wrapper = BadgeNotification::Unwrap<BadgeNotification>(info.This());

      try 
      {
        ::Platform::IBox<::Windows::Foundation::DateTime>^ result = wrapper->_instance->ExpirationTime;
        return scope.Close(result ? NodeRT::Utils::DateTimeToJS(result->Value): static_cast<Handle<Value>>(Undefined()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ExpirationTimeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsDate())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeNotification^>(info.This()))
      {
        return;
      }

      BadgeNotification *wrapper = BadgeNotification::Unwrap<BadgeNotification>(info.This());

      try 
      {
        
        ::Platform::IBox<::Windows::Foundation::DateTime>^ winRtValue = ref new ::Platform::Box<::Windows::Foundation::DateTime>(NodeRT::Utils::DateTimeFromJSDate(value));

        wrapper->_instance->ExpirationTime = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContentGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      BadgeNotification *wrapper = BadgeNotification::Unwrap<BadgeNotification>(info.This());

      try 
      {
        ::Windows::Data::Xml::Dom::XmlDocument^ result = wrapper->_instance->Content;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Data.Xml.Dom", "XmlDocument", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::BadgeNotification^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapBadgeNotification(::Windows::UI::Notifications::BadgeNotification^ wintRtInstance);
    friend ::Windows::UI::Notifications::BadgeNotification^ UnwrapBadgeNotification(Handle<Value> value);
    friend bool IsBadgeNotificationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> BadgeNotification::s_constructorTemplate;

  v8::Handle<v8::Value> WrapBadgeNotification(::Windows::UI::Notifications::BadgeNotification^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(BadgeNotification::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::BadgeNotification^ UnwrapBadgeNotification(Handle<Value> value)
  {
     return BadgeNotification::Unwrap<BadgeNotification>(value.As<Object>())->_instance;
  }

  void InitBadgeNotification(Handle<Object> exports)
  {
    BadgeNotification::Init(exports);
  }

  class ToastNotifier : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ToastNotifier"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("show"), FunctionTemplate::New(Show)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("hide"), FunctionTemplate::New(Hide)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addToSchedule"), FunctionTemplate::New(AddToSchedule)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeFromSchedule"), FunctionTemplate::New(RemoveFromSchedule)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("getScheduledToastNotifications"), FunctionTemplate::New(GetScheduledToastNotifications)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("setting"), SettingGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ToastNotifier"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ToastNotifier(::Windows::UI::Notifications::ToastNotifier^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::ToastNotifier^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotifier^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::ToastNotifier^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ToastNotifier *wrapperInstance = new ToastNotifier(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> Show(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotifier^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ToastNotifier *wrapper = ToastNotifier::Unwrap<ToastNotifier>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args[0]))
      {
        try
        {
          ::Windows::UI::Notifications::ToastNotification^ arg0 = UnwrapToastNotification(args[0]);
          
          wrapper->_instance->Show(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> Hide(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotifier^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ToastNotifier *wrapper = ToastNotifier::Unwrap<ToastNotifier>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args[0]))
      {
        try
        {
          ::Windows::UI::Notifications::ToastNotification^ arg0 = UnwrapToastNotification(args[0]);
          
          wrapper->_instance->Hide(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> AddToSchedule(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotifier^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ToastNotifier *wrapper = ToastNotifier::Unwrap<ToastNotifier>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(args[0]))
      {
        try
        {
          ::Windows::UI::Notifications::ScheduledToastNotification^ arg0 = UnwrapScheduledToastNotification(args[0]);
          
          wrapper->_instance->AddToSchedule(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> RemoveFromSchedule(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotifier^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ToastNotifier *wrapper = ToastNotifier::Unwrap<ToastNotifier>(args.This());

      if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(args[0]))
      {
        try
        {
          ::Windows::UI::Notifications::ScheduledToastNotification^ arg0 = UnwrapScheduledToastNotification(args[0]);
          
          wrapper->_instance->RemoveFromSchedule(arg0);
          return scope.Close(Undefined());   
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetScheduledToastNotifications(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotifier^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ToastNotifier *wrapper = ToastNotifier::Unwrap<ToastNotifier>(args.This());

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::Foundation::Collections::IVectorView<::Windows::UI::Notifications::ScheduledToastNotification^>^ result;
          result = wrapper->_instance->GetScheduledToastNotifications();
          return scope.Close(NodeRT::Collections::VectorViewWrapper<::Windows::UI::Notifications::ScheduledToastNotification^>::CreateVectorViewWrapper(result, 
            [](::Windows::UI::Notifications::ScheduledToastNotification^ val) -> Handle<Value> {
              return WrapScheduledToastNotification(val);
            },
            [](Handle<Value> value) -> bool {
              return NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(value);
            },
            [](Handle<Value> value) -> ::Windows::UI::Notifications::ScheduledToastNotification^ {
              return UnwrapScheduledToastNotification(value);
            }
          ));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> SettingGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotifier^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ToastNotifier *wrapper = ToastNotifier::Unwrap<ToastNotifier>(info.This());

      try 
      {
        ::Windows::UI::Notifications::NotificationSetting result = wrapper->_instance->Setting;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::ToastNotifier^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapToastNotifier(::Windows::UI::Notifications::ToastNotifier^ wintRtInstance);
    friend ::Windows::UI::Notifications::ToastNotifier^ UnwrapToastNotifier(Handle<Value> value);
    friend bool IsToastNotifierWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ToastNotifier::s_constructorTemplate;

  v8::Handle<v8::Value> WrapToastNotifier(::Windows::UI::Notifications::ToastNotifier^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ToastNotifier::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::ToastNotifier^ UnwrapToastNotifier(Handle<Value> value)
  {
     return ToastNotifier::Unwrap<ToastNotifier>(value.As<Object>())->_instance;
  }

  void InitToastNotifier(Handle<Object> exports)
  {
    ToastNotifier::Init(exports);
  }

  class ToastNotification : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ToastNotification"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                        
      Local<Function> addListenerFunc = FunctionTemplate::New(AddListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("addListener"), addListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("on"), addListenerFunc);
      Local<Function> removeListenerFunc = FunctionTemplate::New(RemoveListener)->GetFunction();
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("removeListener"), removeListenerFunc);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("off"), removeListenerFunc);
            
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("expirationTime"), ExpirationTimeGetter, ExpirationTimeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("content"), ContentGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ToastNotification"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ToastNotification(::Windows::UI::Notifications::ToastNotification^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::ToastNotification^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::ToastNotification^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Data::Xml::Dom::XmlDocument^>(args[0]))
      {
        try
        {
          ::Windows::Data::Xml::Dom::XmlDocument^ arg0 = dynamic_cast<::Windows::Data::Xml::Dom::XmlDocument^>(NodeRT::Utils::GetObjectInstance(args[0]));
          
          winRtInstance = ref new ::Windows::UI::Notifications::ToastNotification(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ToastNotification *wrapperInstance = new ToastNotification(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> ExpirationTimeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(info.This());

      try 
      {
        ::Platform::IBox<::Windows::Foundation::DateTime>^ result = wrapper->_instance->ExpirationTime;
        return scope.Close(result ? NodeRT::Utils::DateTimeToJS(result->Value): static_cast<Handle<Value>>(Undefined()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ExpirationTimeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsDate())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(info.This()))
      {
        return;
      }

      ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(info.This());

      try 
      {
        
        ::Platform::IBox<::Windows::Foundation::DateTime>^ winRtValue = ref new ::Platform::Box<::Windows::Foundation::DateTime>(NodeRT::Utils::DateTimeFromJSDate(value));

        wrapper->_instance->ExpirationTime = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContentGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(info.This());

      try 
      {
        ::Windows::Data::Xml::Dom::XmlDocument^ result = wrapper->_instance->Content;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Data.Xml.Dom", "XmlDocument", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


    static v8::Handle<v8::Value> AddListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected arguments are eventName(string),callback(function)")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;
      
      Local<Function> callback = args[1].As<Function>();
      
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      if (NodeRT::Utils::CaseInsenstiveEquals(L"activated", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->Activated::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::UI::Notifications::ToastNotification^, ::Platform::Object^>(
            [callbackObjPtr](::Windows::UI::Notifications::ToastNotification^ arg0, ::Platform::Object^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapToastNotification(arg0);
                Handle<Value> wrappedArg1 = CreateOpaqueWrapper(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"dismissed", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->Dismissed::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::UI::Notifications::ToastNotification^, ::Windows::UI::Notifications::ToastDismissedEventArgs^>(
            [callbackObjPtr](::Windows::UI::Notifications::ToastNotification^ arg0, ::Windows::UI::Notifications::ToastDismissedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapToastNotification(arg0);
                Handle<Value> wrappedArg1 = WrapToastDismissedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else if (NodeRT::Utils::CaseInsenstiveEquals(L"failed", str))
      {
        if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args.This()))
        {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
          return scope.Close(Undefined());
        }
        ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(args.This());
      
        try
        {
          std::shared_ptr<Persistent<Object>> callbackObjPtr(new Persistent<Object>(Persistent<Object>::New(NodeRT::Utils::CreateCallbackObjectInDomain(callback))), 
            [] (Persistent<Object> *ptr ) {
              NodeUtils::Async::RunOnMain([ptr]() {
                ptr->Dispose();
                delete ptr;
            });
          });

          registrationToken = wrapper->_instance->Failed::add(
            ref new ::Windows::Foundation::TypedEventHandler<::Windows::UI::Notifications::ToastNotification^, ::Windows::UI::Notifications::ToastFailedEventArgs^>(
            [callbackObjPtr](::Windows::UI::Notifications::ToastNotification^ arg0, ::Windows::UI::Notifications::ToastFailedEventArgs^ arg1) {
              NodeUtils::Async::RunOnMain([callbackObjPtr , arg0, arg1]() {
                TryCatch tryCatch;
              
                Handle<Value> error;

                Handle<Value> wrappedArg0 = WrapToastNotification(arg0);
                Handle<Value> wrappedArg1 = WrapToastFailedEventArgs(arg1);

                if (tryCatch.HasCaught())
                {
                  error = tryCatch.Exception()->ToObject();
                }
                else 
                {
                  error = Undefined();
                }


                if (wrappedArg0.IsEmpty()) wrappedArg0 = Undefined();
                if (wrappedArg1.IsEmpty()) wrappedArg1 = Undefined();

                Handle<Value> args[] = { wrappedArg0, wrappedArg1 };
                NodeRT::Utils::CallCallbackInDomain(*callbackObjPtr, _countof(args), args);
              });
            })
          );
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }

      }
      else 
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
          tokenMap = Object::New();
          callback->SetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME), tokenMap);
      }

      tokenMap.As<Object>()->Set(args[1], CreateOpaqueWrapper(::Windows::Foundation::PropertyValue::CreateInt64(registrationToken.Value)));
                
      return scope.Close(Undefined());
    }

    static v8::Handle<v8::Value> RemoveListener(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsFunction())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"wrong arguments, expected a string and a callback")));
        return scope.Close(Undefined());
      }

      String::Value eventName(args[0]);
      auto str = *eventName;

      if ((NodeRT::Utils::CaseInsenstiveEquals(L"activated", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"dismissed", str)) &&(NodeRT::Utils::CaseInsenstiveEquals(L"failed", str)))
      {
        ThrowException(Exception::Error(String::Concat(NodeRT::Utils::NewString(L"given event name isn't supported: "), args[0].As<String>())));
        return scope.Close(Undefined());
      }

      Local<Function> callback = args[1].As<Function>();
      Handle<Value> tokenMap = callback->GetHiddenValue(String::NewSymbol(REGISTRATION_TOKEN_MAP_PROPERTY_NAME));
                
      if (tokenMap.IsEmpty() || tokenMap->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      Handle<Value> opaqueWrapperObj =  tokenMap.As<Object>()->Get(args[1]);

      if (opaqueWrapperObj.IsEmpty() || opaqueWrapperObj->Equals(Undefined()))
      {
        return scope.Close(Undefined());
      }

      OpaqueWrapper *opaqueWrapper = OpaqueWrapper::Unwrap<OpaqueWrapper>(opaqueWrapperObj.As<Object>());
            
      long long tokenValue = (long long) opaqueWrapper->GetObjectInstance();
      ::Windows::Foundation::EventRegistrationToken registrationToken;
      registrationToken.Value = tokenValue;
        
      try 
      {
        if (NodeRT::Utils::CaseInsenstiveEquals(L"activated", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(args.This());
          wrapper->_instance->Activated::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"dismissed", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(args.This());
          wrapper->_instance->Dismissed::remove(registrationToken);
        }
        else if (NodeRT::Utils::CaseInsenstiveEquals(L"failed", str))
        {
          if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotification^>(args.This()))
          {
            ThrowException(Exception::Error(NodeRT::Utils::NewString(L"The caller of this method isn't of the expected type or internal WinRt object was disposed")));
            return scope.Close(Undefined());
          }
          ToastNotification *wrapper = ToastNotification::Unwrap<ToastNotification>(args.This());
          wrapper->_instance->Failed::remove(registrationToken);
        }
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }

      tokenMap.As<Object>()->Delete(args[0].As<String>());

      return scope.Close(Undefined());
    }
  private:
    ::Windows::UI::Notifications::ToastNotification^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapToastNotification(::Windows::UI::Notifications::ToastNotification^ wintRtInstance);
    friend ::Windows::UI::Notifications::ToastNotification^ UnwrapToastNotification(Handle<Value> value);
    friend bool IsToastNotificationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ToastNotification::s_constructorTemplate;

  v8::Handle<v8::Value> WrapToastNotification(::Windows::UI::Notifications::ToastNotification^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ToastNotification::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::ToastNotification^ UnwrapToastNotification(Handle<Value> value)
  {
     return ToastNotification::Unwrap<ToastNotification>(value.As<Object>())->_instance;
  }

  void InitToastNotification(Handle<Object> exports)
  {
    ToastNotification::Init(exports);
  }

  class ScheduledToastNotification : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ScheduledToastNotification"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("id"), IdGetter, IdSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("content"), ContentGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("deliveryTime"), DeliveryTimeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maximumSnoozeCount"), MaximumSnoozeCountGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("snoozeInterval"), SnoozeIntervalGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ScheduledToastNotification"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ScheduledToastNotification(::Windows::UI::Notifications::ScheduledToastNotification^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::ScheduledToastNotification^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::ScheduledToastNotification^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 2
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Data::Xml::Dom::XmlDocument^>(args[0])
        && args[1]->IsDate())
      {
        try
        {
          ::Windows::Data::Xml::Dom::XmlDocument^ arg0 = dynamic_cast<::Windows::Data::Xml::Dom::XmlDocument^>(NodeRT::Utils::GetObjectInstance(args[0]));
          ::Windows::Foundation::DateTime arg1 = NodeRT::Utils::DateTimeFromJSDate(args[1]);
          
          winRtInstance = ref new ::Windows::UI::Notifications::ScheduledToastNotification(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 4
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Data::Xml::Dom::XmlDocument^>(args[0])
        && args[1]->IsDate()
        && args[2]->IsNumber()
        && args[3]->IsUint32())
      {
        try
        {
          ::Windows::Data::Xml::Dom::XmlDocument^ arg0 = dynamic_cast<::Windows::Data::Xml::Dom::XmlDocument^>(NodeRT::Utils::GetObjectInstance(args[0]));
          ::Windows::Foundation::DateTime arg1 = NodeRT::Utils::DateTimeFromJSDate(args[1]);
          ::Windows::Foundation::TimeSpan arg2 = NodeRT::Utils::TimeSpanFromMilli(args[2]->IntegerValue());
          unsigned int arg3 = static_cast<unsigned int>(args[3]->IntegerValue());
          
          winRtInstance = ref new ::Windows::UI::Notifications::ScheduledToastNotification(arg0,arg1,arg2,arg3);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ScheduledToastNotification *wrapperInstance = new ScheduledToastNotification(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> IdGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledToastNotification *wrapper = ScheduledToastNotification::Unwrap<ScheduledToastNotification>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Id;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void IdSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsString())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(info.This()))
      {
        return;
      }

      ScheduledToastNotification *wrapper = ScheduledToastNotification::Unwrap<ScheduledToastNotification>(info.This());

      try 
      {
        
        Platform::String^ winRtValue = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(value)));

        wrapper->_instance->Id = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContentGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledToastNotification *wrapper = ScheduledToastNotification::Unwrap<ScheduledToastNotification>(info.This());

      try 
      {
        ::Windows::Data::Xml::Dom::XmlDocument^ result = wrapper->_instance->Content;
        return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Data.Xml.Dom", "XmlDocument", result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DeliveryTimeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledToastNotification *wrapper = ScheduledToastNotification::Unwrap<ScheduledToastNotification>(info.This());

      try 
      {
        ::Windows::Foundation::DateTime result = wrapper->_instance->DeliveryTime;
        return scope.Close(NodeRT::Utils::DateTimeToJS(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaximumSnoozeCountGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledToastNotification *wrapper = ScheduledToastNotification::Unwrap<ScheduledToastNotification>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->MaximumSnoozeCount;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SnoozeIntervalGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ScheduledToastNotification^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ScheduledToastNotification *wrapper = ScheduledToastNotification::Unwrap<ScheduledToastNotification>(info.This());

      try 
      {
        ::Platform::IBox<::Windows::Foundation::TimeSpan>^ result = wrapper->_instance->SnoozeInterval;
        return scope.Close(result ? Number::New(result->Value.Duration/10000.0): static_cast<Handle<Value>>(Undefined()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::ScheduledToastNotification^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapScheduledToastNotification(::Windows::UI::Notifications::ScheduledToastNotification^ wintRtInstance);
    friend ::Windows::UI::Notifications::ScheduledToastNotification^ UnwrapScheduledToastNotification(Handle<Value> value);
    friend bool IsScheduledToastNotificationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ScheduledToastNotification::s_constructorTemplate;

  v8::Handle<v8::Value> WrapScheduledToastNotification(::Windows::UI::Notifications::ScheduledToastNotification^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ScheduledToastNotification::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::ScheduledToastNotification^ UnwrapScheduledToastNotification(Handle<Value> value)
  {
     return ScheduledToastNotification::Unwrap<ScheduledToastNotification>(value.As<Object>())->_instance;
  }

  void InitScheduledToastNotification(Handle<Object> exports)
  {
    ScheduledToastNotification::Init(exports);
  }

  class ToastDismissedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ToastDismissedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("reason"), ReasonGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ToastDismissedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ToastDismissedEventArgs(::Windows::UI::Notifications::ToastDismissedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::ToastDismissedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastDismissedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::ToastDismissedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ToastDismissedEventArgs *wrapperInstance = new ToastDismissedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> ReasonGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastDismissedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ToastDismissedEventArgs *wrapper = ToastDismissedEventArgs::Unwrap<ToastDismissedEventArgs>(info.This());

      try 
      {
        ::Windows::UI::Notifications::ToastDismissalReason result = wrapper->_instance->Reason;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::ToastDismissedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapToastDismissedEventArgs(::Windows::UI::Notifications::ToastDismissedEventArgs^ wintRtInstance);
    friend ::Windows::UI::Notifications::ToastDismissedEventArgs^ UnwrapToastDismissedEventArgs(Handle<Value> value);
    friend bool IsToastDismissedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ToastDismissedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapToastDismissedEventArgs(::Windows::UI::Notifications::ToastDismissedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ToastDismissedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::ToastDismissedEventArgs^ UnwrapToastDismissedEventArgs(Handle<Value> value)
  {
     return ToastDismissedEventArgs::Unwrap<ToastDismissedEventArgs>(value.As<Object>())->_instance;
  }

  void InitToastDismissedEventArgs(Handle<Object> exports)
  {
    ToastDismissedEventArgs::Init(exports);
  }

  class ToastFailedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ToastFailedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("errorCode"), ErrorCodeGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ToastFailedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ToastFailedEventArgs(::Windows::UI::Notifications::ToastFailedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::ToastFailedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastFailedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::ToastFailedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ToastFailedEventArgs *wrapperInstance = new ToastFailedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> ErrorCodeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastFailedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ToastFailedEventArgs *wrapper = ToastFailedEventArgs::Unwrap<ToastFailedEventArgs>(info.This());

      try 
      {
        ::Windows::Foundation::HResult result = wrapper->_instance->ErrorCode;
        return scope.Close(Integer::New(result.Value));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::ToastFailedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapToastFailedEventArgs(::Windows::UI::Notifications::ToastFailedEventArgs^ wintRtInstance);
    friend ::Windows::UI::Notifications::ToastFailedEventArgs^ UnwrapToastFailedEventArgs(Handle<Value> value);
    friend bool IsToastFailedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ToastFailedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapToastFailedEventArgs(::Windows::UI::Notifications::ToastFailedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ToastFailedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::ToastFailedEventArgs^ UnwrapToastFailedEventArgs(Handle<Value> value)
  {
     return ToastFailedEventArgs::Unwrap<ToastFailedEventArgs>(value.As<Object>())->_instance;
  }

  void InitToastFailedEventArgs(Handle<Object> exports)
  {
    ToastFailedEventArgs::Init(exports);
  }

  class TileUpdateManager : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("TileUpdateManager"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("createTileUpdaterForApplication"), FunctionTemplate::New(CreateTileUpdaterForApplication)->GetFunction());
      constructor->Set(String::NewSymbol("createTileUpdaterForSecondaryTile"), FunctionTemplate::New(CreateTileUpdaterForSecondaryTile)->GetFunction());
      constructor->Set(String::NewSymbol("getTemplateContent"), FunctionTemplate::New(GetTemplateContent)->GetFunction());

      exports->Set(String::NewSymbol("TileUpdateManager"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    TileUpdateManager(::Windows::UI::Notifications::TileUpdateManager^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::TileUpdateManager^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::TileUpdateManager^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::TileUpdateManager^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      TileUpdateManager *wrapperInstance = new TileUpdateManager(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  


    static Handle<Value> CreateTileUpdaterForApplication(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::UI::Notifications::TileUpdater^ result;
          result = ::Windows::UI::Notifications::TileUpdateManager::CreateTileUpdaterForApplication();
          return scope.Close(WrapTileUpdater(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          ::Windows::UI::Notifications::TileUpdater^ result;
          result = ::Windows::UI::Notifications::TileUpdateManager::CreateTileUpdaterForApplication(arg0);
          return scope.Close(WrapTileUpdater(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateTileUpdaterForSecondaryTile(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          ::Windows::UI::Notifications::TileUpdater^ result;
          result = ::Windows::UI::Notifications::TileUpdateManager::CreateTileUpdaterForSecondaryTile(arg0);
          return scope.Close(WrapTileUpdater(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetTemplateContent(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::UI::Notifications::TileTemplateType arg0 = static_cast<::Windows::UI::Notifications::TileTemplateType>(args[0]->Int32Value());
          
          ::Windows::Data::Xml::Dom::XmlDocument^ result;
          result = ::Windows::UI::Notifications::TileUpdateManager::GetTemplateContent(arg0);
          return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Data.Xml.Dom", "XmlDocument", result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



  private:
    ::Windows::UI::Notifications::TileUpdateManager^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapTileUpdateManager(::Windows::UI::Notifications::TileUpdateManager^ wintRtInstance);
    friend ::Windows::UI::Notifications::TileUpdateManager^ UnwrapTileUpdateManager(Handle<Value> value);
    friend bool IsTileUpdateManagerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> TileUpdateManager::s_constructorTemplate;

  v8::Handle<v8::Value> WrapTileUpdateManager(::Windows::UI::Notifications::TileUpdateManager^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(TileUpdateManager::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::TileUpdateManager^ UnwrapTileUpdateManager(Handle<Value> value)
  {
     return TileUpdateManager::Unwrap<TileUpdateManager>(value.As<Object>())->_instance;
  }

  void InitTileUpdateManager(Handle<Object> exports)
  {
    TileUpdateManager::Init(exports);
  }

  class BadgeUpdateManager : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("BadgeUpdateManager"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("createBadgeUpdaterForApplication"), FunctionTemplate::New(CreateBadgeUpdaterForApplication)->GetFunction());
      constructor->Set(String::NewSymbol("createBadgeUpdaterForSecondaryTile"), FunctionTemplate::New(CreateBadgeUpdaterForSecondaryTile)->GetFunction());
      constructor->Set(String::NewSymbol("getTemplateContent"), FunctionTemplate::New(GetTemplateContent)->GetFunction());

      exports->Set(String::NewSymbol("BadgeUpdateManager"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    BadgeUpdateManager(::Windows::UI::Notifications::BadgeUpdateManager^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::BadgeUpdateManager^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::BadgeUpdateManager^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::BadgeUpdateManager^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      BadgeUpdateManager *wrapperInstance = new BadgeUpdateManager(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  


    static Handle<Value> CreateBadgeUpdaterForApplication(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::UI::Notifications::BadgeUpdater^ result;
          result = ::Windows::UI::Notifications::BadgeUpdateManager::CreateBadgeUpdaterForApplication();
          return scope.Close(WrapBadgeUpdater(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          ::Windows::UI::Notifications::BadgeUpdater^ result;
          result = ::Windows::UI::Notifications::BadgeUpdateManager::CreateBadgeUpdaterForApplication(arg0);
          return scope.Close(WrapBadgeUpdater(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> CreateBadgeUpdaterForSecondaryTile(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          ::Windows::UI::Notifications::BadgeUpdater^ result;
          result = ::Windows::UI::Notifications::BadgeUpdateManager::CreateBadgeUpdaterForSecondaryTile(arg0);
          return scope.Close(WrapBadgeUpdater(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetTemplateContent(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::UI::Notifications::BadgeTemplateType arg0 = static_cast<::Windows::UI::Notifications::BadgeTemplateType>(args[0]->Int32Value());
          
          ::Windows::Data::Xml::Dom::XmlDocument^ result;
          result = ::Windows::UI::Notifications::BadgeUpdateManager::GetTemplateContent(arg0);
          return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Data.Xml.Dom", "XmlDocument", result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



  private:
    ::Windows::UI::Notifications::BadgeUpdateManager^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapBadgeUpdateManager(::Windows::UI::Notifications::BadgeUpdateManager^ wintRtInstance);
    friend ::Windows::UI::Notifications::BadgeUpdateManager^ UnwrapBadgeUpdateManager(Handle<Value> value);
    friend bool IsBadgeUpdateManagerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> BadgeUpdateManager::s_constructorTemplate;

  v8::Handle<v8::Value> WrapBadgeUpdateManager(::Windows::UI::Notifications::BadgeUpdateManager^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(BadgeUpdateManager::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::BadgeUpdateManager^ UnwrapBadgeUpdateManager(Handle<Value> value)
  {
     return BadgeUpdateManager::Unwrap<BadgeUpdateManager>(value.As<Object>())->_instance;
  }

  void InitBadgeUpdateManager(Handle<Object> exports)
  {
    BadgeUpdateManager::Init(exports);
  }

  class ToastNotificationManager : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ToastNotificationManager"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("createToastNotifier"), FunctionTemplate::New(CreateToastNotifier)->GetFunction());
      constructor->Set(String::NewSymbol("getTemplateContent"), FunctionTemplate::New(GetTemplateContent)->GetFunction());

      exports->Set(String::NewSymbol("ToastNotificationManager"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ToastNotificationManager(::Windows::UI::Notifications::ToastNotificationManager^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::ToastNotificationManager^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastNotificationManager^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::ToastNotificationManager^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ToastNotificationManager *wrapperInstance = new ToastNotificationManager(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  


    static Handle<Value> CreateToastNotifier(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          ::Windows::UI::Notifications::ToastNotifier^ result;
          result = ::Windows::UI::Notifications::ToastNotificationManager::CreateToastNotifier();
          return scope.Close(WrapToastNotifier(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else if (args.Length() == 1
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          ::Windows::UI::Notifications::ToastNotifier^ result;
          result = ::Windows::UI::Notifications::ToastNotificationManager::CreateToastNotifier(arg0);
          return scope.Close(WrapToastNotifier(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> GetTemplateContent(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::UI::Notifications::ToastTemplateType arg0 = static_cast<::Windows::UI::Notifications::ToastTemplateType>(args[0]->Int32Value());
          
          ::Windows::Data::Xml::Dom::XmlDocument^ result;
          result = ::Windows::UI::Notifications::ToastNotificationManager::GetTemplateContent(arg0);
          return scope.Close(NodeRT::Utils::CreateExternalWinRTObject("Windows.Data.Xml.Dom", "XmlDocument", result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



  private:
    ::Windows::UI::Notifications::ToastNotificationManager^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapToastNotificationManager(::Windows::UI::Notifications::ToastNotificationManager^ wintRtInstance);
    friend ::Windows::UI::Notifications::ToastNotificationManager^ UnwrapToastNotificationManager(Handle<Value> value);
    friend bool IsToastNotificationManagerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ToastNotificationManager::s_constructorTemplate;

  v8::Handle<v8::Value> WrapToastNotificationManager(::Windows::UI::Notifications::ToastNotificationManager^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ToastNotificationManager::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::ToastNotificationManager^ UnwrapToastNotificationManager(Handle<Value> value)
  {
     return ToastNotificationManager::Unwrap<ToastNotificationManager>(value.As<Object>())->_instance;
  }

  void InitToastNotificationManager(Handle<Object> exports)
  {
    ToastNotificationManager::Init(exports);
  }

  class ToastActivatedEventArgs : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ToastActivatedEventArgs"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("arguments"), ArgumentsGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ToastActivatedEventArgs"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ToastActivatedEventArgs(::Windows::UI::Notifications::ToastActivatedEventArgs^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::UI::Notifications::ToastActivatedEventArgs^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastActivatedEventArgs^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::UI::Notifications::ToastActivatedEventArgs^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ToastActivatedEventArgs *wrapperInstance = new ToastActivatedEventArgs(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> ArgumentsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::UI::Notifications::ToastActivatedEventArgs^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ToastActivatedEventArgs *wrapper = ToastActivatedEventArgs::Unwrap<ToastActivatedEventArgs>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->Arguments;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::UI::Notifications::ToastActivatedEventArgs^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapToastActivatedEventArgs(::Windows::UI::Notifications::ToastActivatedEventArgs^ wintRtInstance);
    friend ::Windows::UI::Notifications::ToastActivatedEventArgs^ UnwrapToastActivatedEventArgs(Handle<Value> value);
    friend bool IsToastActivatedEventArgsWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ToastActivatedEventArgs::s_constructorTemplate;

  v8::Handle<v8::Value> WrapToastActivatedEventArgs(::Windows::UI::Notifications::ToastActivatedEventArgs^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ToastActivatedEventArgs::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::UI::Notifications::ToastActivatedEventArgs^ UnwrapToastActivatedEventArgs(Handle<Value> value)
  {
     return ToastActivatedEventArgs::Unwrap<ToastActivatedEventArgs>(value.As<Object>())->_instance;
  }

  void InitToastActivatedEventArgs(Handle<Object> exports)
  {
    ToastActivatedEventArgs::Init(exports);
  }

} } } } 

void init(Handle<Object> exports)
{
  if (FAILED(CoInitializeEx(nullptr, COINIT_MULTITHREADED)))
  {
    ThrowException(v8::Exception::Error(NodeRT::Utils::NewString(L"error in CoInitializeEx()")));
    return;
  }
  
  NodeRT::Windows::UI::Notifications::InitNotificationSettingEnum(exports);
  NodeRT::Windows::UI::Notifications::InitToastDismissalReasonEnum(exports);
  NodeRT::Windows::UI::Notifications::InitBadgeTemplateTypeEnum(exports);
  NodeRT::Windows::UI::Notifications::InitTileTemplateTypeEnum(exports);
  NodeRT::Windows::UI::Notifications::InitToastTemplateTypeEnum(exports);
  NodeRT::Windows::UI::Notifications::InitPeriodicUpdateRecurrenceEnum(exports);
  NodeRT::Windows::UI::Notifications::InitTileUpdater(exports);
  NodeRT::Windows::UI::Notifications::InitTileNotification(exports);
  NodeRT::Windows::UI::Notifications::InitScheduledTileNotification(exports);
  NodeRT::Windows::UI::Notifications::InitBadgeUpdater(exports);
  NodeRT::Windows::UI::Notifications::InitBadgeNotification(exports);
  NodeRT::Windows::UI::Notifications::InitToastNotifier(exports);
  NodeRT::Windows::UI::Notifications::InitToastNotification(exports);
  NodeRT::Windows::UI::Notifications::InitScheduledToastNotification(exports);
  NodeRT::Windows::UI::Notifications::InitToastDismissedEventArgs(exports);
  NodeRT::Windows::UI::Notifications::InitToastFailedEventArgs(exports);
  NodeRT::Windows::UI::Notifications::InitTileUpdateManager(exports);
  NodeRT::Windows::UI::Notifications::InitBadgeUpdateManager(exports);
  NodeRT::Windows::UI::Notifications::InitToastNotificationManager(exports);
  NodeRT::Windows::UI::Notifications::InitToastActivatedEventArgs(exports);

  NodeRT::Utils::RegisterNameSpace("Windows.UI.Notifications", exports);
}


NODE_MODULE(binding, init)